import 'package:openfoodfacts/openfoodfacts.dart';

class ProductImageData {
  const ProductImageData({
    required this.imageField,
    this.imageUrl,
  });

  factory ProductImageData.from(ProductImage image, String barcode) {
    return ProductImageData(
      imageField: image.field,
      imageUrl: ImageHelper.buildUrl(barcode, image),
    );
  }

  final ImageField imageField;
  final String? imageUrl;

  /// Try to convert [imageUrl] to specified [size].
  /// Note that url for specified [size] might not exist on API.
  String? getImageUrl(ImageSize size) {
    final String? imageUrl = this.imageUrl;
    if (imageUrl == null) {
      return null;
    }

    const String SEPARATOR = '.';

    final int extensionIndex = imageUrl.lastIndexOf(SEPARATOR);
    if (extensionIndex < 0) {
      return null;
    }

    final int sizeIndex = imageUrl.lastIndexOf(SEPARATOR, extensionIndex - 1);
    if (sizeIndex < 0) {
      return null;
    }

    final String baseUrl = imageUrl.substring(0, sizeIndex + 1);
    final String number = size.toNumber();
    final String extension =
        imageUrl.substring(extensionIndex, imageUrl.length);
    return baseUrl + number + extension;
  }
}
